@@ -1,7 +1,6 @@
-# Copyright (c) 2016  Richard Plevin
+# Copyright (c) 2016-2021  Richard Plevin
 # See the https://opensource.org/licenses/MIT for license details.
 #
-# TBD: may be obsolete
 
 from pygcam.log import getLogger
 from .McsSubcommandABC import McsSubcommandABC
@@ -16,7 +15,7 @@ def driver(args, tool):
     from ..Database import getDatabase
 
     db = getDatabase()
-    expId = db.createExp(args.expName, args.description)
+    expId = db.createExp(args.expName, parent=args.parent, description=args.description)
     _logger.debug("Added experiment '%s' with id=%d" % (args.expName, expId))
 
 
@@ -33,6 +32,12 @@ class AddExpCommand(McsSubcommandABC):
                             default='No description',
                             help='Add the named experiment to the database.')
 
+        parser.add_argument('-p', '--parent', type=str, required=False,
+                            default=None,
+                            help='''Set the "parent" of this experiment. If adding a policy scenario,
+                            the parent should be the name of the baseline. If adding a baseline, do not
+                            specify a parent.''')
+
         return parser   # for auto-doc generation
 
 
