@@ -178,55 +178,59 @@ class Master(object):
         return totals
 
     def shutdownIdleEngines(self):
-        qstatus = self.client.queue_status()
+        client = self.client
+        qstatus = client.queue_status()
 
         if qstatus.pop(u'unassigned') > 0:
             # some tasks are not yet assigned to engines => not idle
             return
 
-        # TBD: iterate over client.ids instead?
-        # From http://python.6.x6.nabble.com/IPython-User-IPython-parallel-shutdown-unused-engines-td5053278.html#a5053309
-        # # get queue status
-        # qs = rc.queue_status()
-        #
-        # # identify idle engines
-        # idle = []
-        # for eid in rc.ids:
-        #     if qs[eid]['queue'] + qs[eid]['tasks'] == 0:
-        #         idle.append(eid)
-        #
-        # # shut them down
-        # dv = rc.direct_view(idle)
-        # dv.shutdown()
-
-
-        engineCount = len(qstatus) # TBD: any different/better to use len(self.client.ids)?
-        if engineCount == 0:
+        if len(client.ids) == 0:
             _logger.info("No engines are running")
             return
 
-        # Shutdown idle engines if there are no unassigned tasks
-        idleEngines = set([eid for eid, qs in iteritems(qstatus) if
-                isinstance(eid, string_types) and eid.isdigit() and (qs[u'tasks'] + qs[u'queue']) == 0])
+        # From http://python.6.x6.nabble.com/IPython-User-IPython-parallel-shutdown-unused-engines-td5053278.html#a5053309
+        idle = []
+        for eid in client.ids:
+            if qstatus[eid]['queue'] + qstatus[eid]['tasks'] == 0:
+                idle.append(eid)
 
-        if idleEngines:
-            newlyIdle = idleEngines.difference(self.idleEngines)
-            self.idleEngines = self.idleEngines.union(idleEngines)
+        if len(idle):
+            _logger.info('Shutting down %d idle engines', len(idle))
+            _logger.debug('Idle engines: %s', idle)
+            dv = client.direct_view(idle)
 
-            if newlyIdle:
-                _logger.debug('Idle engines: %s', newlyIdle)
-                _logger.info('Shutting down %d idle engines', len(newlyIdle))
+            try:
+                dv.shutdown(block=False)
 
-                try:
-                    self.client.shutdown(targets=list(newlyIdle), block=False)
+            except Exception as e:
+                # report it, then ignore the error
+                _logger.debug("shutdownIdleEngines: %s", e)
 
-                    # TBD?
-                    # dv = self.client.direct_view(list(newlyIdle))
-                    # dv.shutdown(block=False)
 
-                except Exception as e:
-                    _logger.debug("shutdownIdleEngines: %s", e)
-                    # ignore the error
+        # Deprecated: this wasn't working.
+        # # Shutdown idle engines if there are no unassigned tasks
+        # idleEngines = set([eid for eid, qs in iteritems(qstatus) if
+        #         isinstance(eid, string_types) and eid.isdigit() and (qs[u'tasks'] + qs[u'queue']) == 0])
+        #
+        # if idleEngines:
+        #     newlyIdle = idleEngines.difference(self.idleEngines)
+        #     self.idleEngines = self.idleEngines.union(idleEngines)
+        #
+        #     if newlyIdle:
+        #         _logger.debug('Idle engines: %s', newlyIdle)
+        #         _logger.info('Shutting down %d idle engines', len(newlyIdle))
+        #
+        #         try:
+        #             client.shutdown(targets=list(newlyIdle), block=False)
+        #
+        #             # TBD?
+        #             # dv = client.direct_view(list(newlyIdle))
+        #             # dv.shutdown(block=False)
+        #
+        #         except Exception as e:
+        #             _logger.debug("shutdownIdleEngines: %s", e)
+        #             # ignore the error
 
     def createRuns(self, simId, scenario, trialNums):
         '''
@@ -574,12 +578,13 @@ class Master(object):
 
         asyncResults = []
 
-        # TBD: Use "self.client[:]" for a direct view rather than load-balanced view
-        # TBD: This should pack nodes rather than spreading the load, which makes it easier
-        # TBD: to shutdown idle nodes, rather than having nodes running only a single task.
-        # view = None if runLocal else self.client[:]
+        # Use "self.client[:]" for a direct view rather than load-balanced view
+        # This should pack nodes rather than spreading the load, which makes it easier
+        # to shutdown idle nodes, rather than having nodes running only a single task.
+        #view = None if runLocal else self.client.direct_view()
+        # TBD: This didn't work either... debug it.
 
-        view = None if runLocal else self.client.load_balanced_view() # retries=2)
+        view = None if runLocal else self.client.load_balanced_view()
 
         db = getDatabase()
         exps = {e.expName: e.parent for e in db.getExps()}
