@@ -405,6 +405,83 @@ def _findOrCreateQueryFile(title, queryPath, regions, outputDir=None, tmpFiles=T
         tree.write(path, xml_declaration=True, encoding="UTF-8", pretty_print=True)
         return path
 
+def path_to_list(arg):
+    # N.B. os.path.pathsep  is ';' on Windows, ':' on Unix
+    return (arg if isinstance(arg, (list, tuple)) else arg.split(os.path.pathsep))
+
+def extractQueries(titles, xmlFiles, delete=True):
+    """
+    Find the named queries in the given XML files and extract them to a tmp batch file.
+    Return the path to the temporary batch file. Note that both the ``titles`` and ``xmlFiles``
+    arguments can be either a list (or tuple) or a delimited string. On Windows, the delimiter
+    is ":", on macOS and Linux, it's ";".
+
+    :param titles: (list of str, or "path" string with one or more filenames) names of queries
+    :param xmlFiles: (list of str, or "path" string with one or more filenames)
+        pathnames of XML files containing queries
+    :param delete: (bool) whether to delete the temporary file (set to False to debug)
+    :return: (str) the pathname to the temporary batch file
+    """
+    titles = path_to_list(titles)
+    xmlFiles = path_to_list(xmlFiles)
+
+    parser = ET.XMLParser(remove_blank_text=True)
+    root = ET.Element("queries")
+    queryXmlPath = getTempFile(suffix='.query.xml', delete=delete)
+
+    for title in titles:
+        found = False
+
+        for xmlFile in xmlFiles:
+            # Look for the query in the XML query file
+            tree = ET.parse(xmlFile, parser=parser)
+            elts = _findQueryByName(tree, title)
+
+            if elts is None or len(elts) == 0:
+                continue # to next item xml file
+
+            _logger.debug(f"Found query '{title}' in {xmlFile}")
+
+            elt = elts[0]
+            aQuery = elt.getparent()
+            root.append(aQuery)
+
+            found = True
+            break
+
+        if not found:
+            raise PygcamException(f"Query '{title}' not found in {xmlFiles}")
+
+    _logger.debug(f"Extracted queries to '{queryXmlPath}'")
+    tree = ET.ElementTree(root)
+    tree.write(queryXmlPath, xml_declaration=True, encoding="UTF-8", pretty_print=True)
+
+    return queryXmlPath
+
+def runExtractedQueries(scenario, titles, xmlFiles, csvPath, xmldb='', delete=True):
+    if titles.endswith('.xml'):
+        # extract the query titles from the XML
+        parser = ET.XMLParser(remove_blank_text=True)
+        tree = ET.parse(titles, parser=parser)
+        elts = tree.findall('//query')
+        titles = [elt.get('name') for elt in elts]
+
+    titles = path_to_list(titles)
+    queryFile = extractQueries(titles, xmlFiles, delete=delete)
+
+    outputDir, csvFile = os.path.split(csvPath)
+
+    batchFileText = BatchQueryTemplate.format(scenario=scenario, queryFile=queryFile,
+                                              csvFile=csvPath, xmldb=xmldb)
+
+    batchFile = getTempFile(suffix='.batch.xml', delete=delete)
+
+    _logger.debug("Creating temporary batch file '%s'", batchFile)
+    saveToFile(batchFileText, filename=batchFile)
+
+    runModelInterface(scenario, outputDir, xmldb=xmldb,
+                      batchFile=batchFile, noDelete=(not delete))
+
 
 BatchQueryTemplate = """<?xml version="1.0"?>
 <!-- WARNING: this file is automatically generated. Manual edits will be overwritten. -->
@@ -554,6 +631,8 @@ def createBatchFile(scenario, queries, xmldb='', queryPath=None, outputDir=None,
         rewriteSets.xml
     :param batchFileIn: (str) the name of a pre-formed batch file to run
     :param batchFileOut: (str) where to write output from batchFileIn, if given
+    :param singleCSV: (bool) if True, write a batch file that puts all results in a
+        single CSV output file.
     :param tmpFiles: (bool) if True temporary files are used and deleted when the
         program exits, otherwise normal files are create in outputDir.
     :param noDelete: (bool) if True, temporary files created by this function are
@@ -840,8 +919,8 @@ def runBatchQuery(scenario, queryName, queryPath, outputDir, xmldb='',
         csvFile = "%s-%s.csv" % (saveAs or mainPart, scenario)    # compute default filename
         csvFile = csvFile.replace(' ', '_')                       # eliminate spaces for convenience
 
-    csvPath = runModelInterface(scenario, outputDir, csvFile, queryFile=queryFile, xmldb=xmldb,
-                                miLogFile=miLogFile, noDelete=noDelete, noRun=noRun)
+    csvPath = runModelInterface(scenario, outputDir, csvFile=csvFile, queryFile=queryFile,
+                                xmldb=xmldb, miLogFile=miLogFile, noDelete=noDelete, noRun=noRun)
     return csvPath
 
 # Deprecated: there no need for this once multiple queries is working
