@@ -376,8 +376,24 @@ class Function(ConfigAction):
         if not method:
             raise SetupException("<function name='%s'>: function doesn't exist or is not callable from XML" % name)
 
-        args = "(%s)" % self.formattedContent if self.formattedContent else "()"
+        args = "(%s)" % self.formattedContent.strip() if self.formattedContent else "()"
         codeStr = "editor.%s%s" % (name, args)
+
+        ######################
+        # TBD: debugging only
+        DEBUG = False
+        if DEBUG:
+            if codeStr.startswith("editor.writePolicyConstraintFile('base-1-"):
+                # TBD: Call it directly so we can debug
+                editor.writePolicyConstraintFile('base-1-constraints.xml', 'Corn Ethanol', 'USA', None, minPrice=-100,
+                                                 csvPath="/Users/rjp/repos/gcam-mcs/analyses/series46/etc/shock_permanence_runs.csv",
+                                                 scenario="base-1")
+            elif codeStr.startswith("editor.writePolicyMarketFile('base-1"):
+                editor.writePolicyMarketFile('base-1-market.xml', 'Corn Ethanol', 'USA', 'ethanol', 'corn ethanol',
+                                             'corn ethanol', range(2020, 2061, 5))
+            return
+        ######################
+
         try:
             eval(codeStr)
         except SyntaxError as e:
@@ -570,26 +586,19 @@ def createXmlEditorSubclass(setupFile, mcsMode=None, cleanXML=True):
 
     return XmlEditorSubclass
 
-
-def scenarioEditor(scenario):
+def scenarioEditor(scenario, baseline='', xmlSrcDir='', refWorkspace='', mcsMode=None,
+                   groupName='', srcGroupDir='', subdir =''):
     setupXml = getParam('GCAM.ScenarioSetupFile')
     editorClass = createXmlEditorSubclass(setupXml, cleanXML=False)
 
-    # we don't need to specify any of these for real since we're just getting the config file
-    baseline = ''
-    xmlSrcDir = ''
-    refWorkspace = ''
-    groupName = ''
-    srcGroupDir = ''
-    subdir = ''
-
     xmlOutputRoot = pathjoin(getParam('GCAM.SandboxDir'), groupName, scenario, normpath=True)
 
     editor = editorClass(baseline, scenario, xmlOutputRoot, xmlSrcDir, refWorkspace,
-                         groupName, srcGroupDir, subdir, cleanXML=False)
+                         groupName, srcGroupDir, subdir, cleanXML=False, mcsMode=mcsMode)
     return editor
 
 def scenarioConfigPath(scenario):
+    # We don't need to specify most of the keyword args when we're just getting the config file
     editor = scenarioEditor(scenario)
     path = editor.cfgPath()
     return path
